Con                        
                                                         
  _CLKMODE = XTAL1 + PLL16X                              
  _XINFREQ = 5_000_000
  
  DYNA  = 0           'dynamixel servo bus i/o
  CAMX  = 1           'dyna servo id x camera platform 
  CAMY  = 2           'dyna servo id y camera platform 
  MARM  = 3           'dyna servo id main arm
  LGRP  = 4           'dyna servo id left gripper 
  RGRP  = 5           'dyna servo id right grippper 
  RSRV  = 12          'right ping servo output                         
  LSRV  = 13          'left ping servo output                           
  RPNG  = 14          'right ping sensor i/o                            
  LPNG  = 15          'left ping sensor i/o                             
  PWMB  = 24          'stingray pwm base output pin
  WSPD  = 80          'stingray wheel speed limit (0-255)
    TX  = 30          'tracking transmit output (to Pi USB0)
    RX  = 31          'tracking receive input   (to Pi USB0)    
    
Var
  
  long x, y, xAxis, yAxis, zAxis
        
Obj
 
  fds    : "FullDuplexSerial"
  dyn    : "DynaComV4"
  srv    : "Servo32v7"
  pid[2] : "PID"
  pwm    : "PWMx8"         
      
Pub Main | lo, ro

  fds.start(RX, TX, 0, 115_200) 
  dyn.start(DYNA, 1_000_000)
  dyn.setsrvspdall(300)
  pwm.start(PWMB, %0000_1111, 2_000) 
  srv.start
  initRobot
  pid[0].init(2.5, 0.0, 0.4, 55.0, 0.0, 0.0, 0.0)
  pid[1].init(2.5, 0.0, 0.4, 55.0, 0.0, 0.0, 0.0)
 
  fds.rxflush     
  repeat
    objectDetect
    zAxis -= 50
    dyn.setsrvpos(MARM, 600 - yAxis << 2)
    lo := -pid[0].calculate(xAxis) 
    ro := pid[1].calculate(xAxis) 
    lo := -WSPD #> lo <# WSPD 
    ro := -WSPD #> ro <# WSPD 
    moveRobot(lo, ro)
               
Pub initRobot                               'set initial robot servo positions

  srv.set(RSRV, 1500)      'right ping face forward
  srv.set(LSRV, 1000)      'left ping face forward
  dyn.setsrvpos(CAMX, 512) 'lower# = cw, higher = ccw 0-1023
  dyn.setsrvpos(CAMY, 490) 'lower# = down, higher# = up
  dyn.setsrvpos(MARM, 370) 'main arm level with ground
  dyn.setsrvpos(LGRP, 550) 'left gripper claw open
  dyn.setsrvpos(RGRP, 490) 'right gripper claw open
  waitcnt(clkfreq*2+cnt)   'delay for servo position change 

Pub moveRobot(leftWheel, rightWheel)      'set the speed of stingray's left and right wheels, -255 max.reverse to 255 max.forward, 0=stop    
                                                                                          
  ifnot leftWheel & $8000_0000                                      
    pwm.duty(PWMB + 1, leftWheel & $ff)                            
    pwm.duty(PWMB, 0)                                                          
  else                                                                             
    pwm.duty(PWMB + 1, 0)                                                      
    pwm.duty(PWMB, (0 - leftWheel) & $ff)                        
                                                                                   
  ifnot rightWheel & $8000_0000                                 
    pwm.duty(PWMB + 2, rightWheel & $ff)                           
    pwm.duty(PWMB + 3, 0)                                                      
  else                                                                             
    pwm.duty(PWMB + 2, 0)                                                      
    pwm.duty(PWMB + 3, (0 - rightWheel) & $ff)                
  
Pub objectDetect | zl, zu                   'object detect tracking info from pi
                                            'xAxis = 0 to 109 horizontal lft/rgt
  repeat                                    'yAxis = 0 to 79 vertical up/dwn
    if fds.rxcheck == "!"                   'zAxis = 0 to word size sm/lg
      if fds.rxtime(10) == "&"  
        xAxis := fds.rxtime(10)   
        yAxis := fds.rxtime(10)   
        zl := fds.rxtime(10)       
        zu := fds.rxtime(10)       
        zAxis := zu << 8 + zl        
        return